--[[
	Copyright 2010 Matthew Mullins (aka Baffler)
	
    This file is part of Proxxy (World of Warcraft addon).

    Proxxy is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Proxxy is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Proxxy.  If not, see <http://www.gnu.org/licenses/>.
]]--

--[[
  * TODO:
	- Line up the Proxxy bar with the Quartz debuff bar a little better
	- Clean up the main options (scattered all about)
	- Orientation option (grow from left to right or vice versa)
	- Default colors need to be fixed
	- Animation (flashing bars when time is running out... "sliding effect" when bars get removed, etc)
	- Character limit on names of spells to prevent overlapping with timer text
	- Add in the non-aura procs/buffs
  * BUGS:
	- The non-timer bars seem to randomly switch places if theres no charges on them (sort alphabetically maybe?)
]]--

Proxxy = LibStub("AceAddon-3.0"):NewAddon("Proxxy", "AceConsole-3.0", "AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):NewLocale("Proxxy", "enUS", true)
local media = LibStub("LibSharedMedia-3.0")

local cpColor = {0,0,1}
local procs = {}

local defaults = {
    profile = {
		Options = {
			ShowIcons = true,
			--IconScale = 1,
			BarHeight = 14,
			BarWidth = 160,
			GapSize = 1,
			AlphaValue = 1,
			FontSize = 12,
			FontType = "Sans Demibold", -- Lucida Sans Demibold
			TextColor = {1,1,1},
			BarTexture = "Smooth",
			AnchorTo = "FREEUP",
			AnchorToXoff = -12,
			AnchorToYoff = 0,
			AnchorPOS = "TOPLEFT",
			GrowDir = 1, -- 1 = up, 0 = down
		}
	}
}

function Proxxy:OnEnable()
	self:RegisterEvent("UNIT_AURA")
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	self:RegisterEvent("UNIT_COMBO_POINTS")
	self:RegisterEvent("PLAYER_TARGET_CHANGED")
end

function Proxxy:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("ProxxyDB", defaults, true)
	if (self.db.profile.custom == nil) then self.db.profile.custom = {} end
	self:SetupAnchor()
	
	procs = self:returnProcList()
	self:setupOptions(procs)
	for spellkey,spell in pairs(self.db.profile.custom) do
		procs[spellkey] = self.db.profile.custom[spellkey]
	end
	
	self:RegisterChatCommand("proxxy", "ChatCommand")
    self:RegisterChatCommand("pt", "ChatCommand")
	self:RegisterChatCommand("procs", "ChatCommand")
end

function Proxxy:ChatCommand(input)
	if not input or input:trim() == "" then
		if Proxxy.optionsFrame ~= nil then
			InterfaceOptionsFrame_OpenToCategory(Proxxy.optionsFrame)
		else
			error("Options have not be setup!")
		end
		--_,_,sIcon = GetSpellInfo(18093)
		--local testbar = self:NewTimerBar("TestBar", "This is a test", 60, GetTime() + 30, sIcon)
	elseif (input == "dock") then
		Proxxy:SetAnchorPoint("QUARTZPLAYER")
	elseif (input == "lock") then
		Proxxy:LockAnchor(true)
	elseif (input == "unlock") then
		Proxxy:LockAnchor(false)
	elseif (input == "show") then
		Proxxy:HideAnchor(true)
	elseif (input == "hide") then
		Proxxy:HideAnchor(false)
	elseif (input == "reset") then
		self.db.profile.Options.AnchorTo = "FREEUP"
		self.db.profile.Options.AnchorPoint = "CENTER"
		self.db.profile.Options.AnchorRelPoint = "CENTER"
		self.db.profile.Options.AnchorXOfs = 0
		self.db.profile.Options.AnchorYOfs = 0
		Proxxy:SetAnchorPoint("FREEUP")
	elseif (input == "test") then
		_,_,sIcon = GetSpellInfo(18093)
		self:NewTimerBar("TestBar1", "TestBar1", 15, GetTime() + 15, {1,0,0}, sIcon, "combatlog")
		self:NewTimerBar("TestBar2", "TestBar2", 10, GetTime() + 10, {1,0,0}, sIcon, "combatlog")
		self:NewTimerBar("TestBar3", "TestBar3", 18, GetTime() + 18, {1,0,0}, sIcon, "combatlog")
		self:NewTimerBar("TestBar4", "TestBar4", 30, GetTime() + 30, {1,0,0}, sIcon, "combatlog")
		self:NewTimerBar("TestBar5", "TestBar5", 25, GetTime() + 25, {1,0,0}, sIcon, "combatlog")
	else
		_,_,sIcon = GetSpellInfo(18093)
		self:NewTimerBar(input, input, 15, GetTime() + 15, {1,0,0}, sIcon, "combatlog")
	end
end

function Proxxy:UNIT_AURA(e, unit)
	if (unit == "player") then
		self:CheckBuffs()
	end	
end

function Proxxy:COMBAT_LOG_EVENT_UNFILTERED(event, ...)
	local timestamp, eventType, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags = select(1, ...)
	if (destGUID == UnitGUID("player")) then --check if the player is receiving the buff
		local spellID, spellname, spellSchool = select(9, ...)
		if (spellID ~= nil) and (spellID ~= "") then spellID = tostring(spellID) end
		
		if (eventType == "SPELL_AURA_APPLIED") or (eventType == "SPELL_AURA_REFRESH") then
			if (procs ~= nil) and (procs.combatlog ~= nil) and (procs.combatlog[spellID] ~= nil) then
				if (eventType == "SPELL_AURA_REFRESH") then spellname = "" end -- set spellname to nothing so it doesnt update the name on the timerbar (used so like Arcane Blast(4) will stay at 4)
				local spell = procs.combatlog[spellID]
				local expr = GetTime() + spell.dur
				local _,_,icon = GetSpellInfo(tonumber(spellID))
				local sdb = self.db.profile.spells
				--print("newTimerBar("..spellID..", "..spellname..", "..spell.dur..", "..expr..", color, "..icon)
				if ((sdb[spellID] == nil) or (sdb[spellID].enabled == nil) or (sdb[spellID].enabled == true))
				then
					self:NewTimerBar(spellID, spellname, spell.dur, expr, spell.color or {1,0,0}, icon, "combatlog")
				end
			end
		-- for spell stacking (so far this is only going to be used for ArcaneBlast debuff.. hmmm
		elseif (eventType == "SPELL_AURA_APPLIED_DOSE") then
			if (procs ~= nil) and (procs.combatlog ~= nil) and (procs.combatlog[spellID] ~= nil) then
				local spellStack = select(13,...)
				local spell = procs.combatlog[spellID]
				local expr = GetTime() + spell.dur
				local _,_,icon = GetSpellInfo(tonumber(spellID))
				local sdb = self.db.profile.spells
				
				if ((sdb[spellID] == nil) or (sdb[spellID].enabled == nil) or (sdb[spellID].enabled == true))
				then
					self:NewTimerBar(spellID, spellname .. " ("..spellStack..")", spell.dur, expr, spell.color or {1,0,0}, icon, "combatlog")
				end
			end
		elseif (eventType == "SPELL_AURA_REMOVED") then
			if (procs ~= nil) and (procs.combatlog ~= nil) and (procs.combatlog[spellID] ~= nil) then
				local timerbars = self:GetBars()
				if (timerbars[spellID] ~= nil) then
					timerbars[spellID]:destroy()
				end
			end
		end
	end	
end

function Proxxy:UNIT_COMBO_POINTS()
	local cp = GetComboPoints("player", "target")
	if (cp > 0) then
		_,_,sIcon = GetSpellInfo(48889)
		self:NewCounterBar("Combo Points", "Combo Points", cp, 5, cpColor, sIcon)
	else
		local timerbars = self:GetBars()
		if (timerbars["Combo Points"] ~= nil) then
			timerbars["Combo Points"]:destroy()
		end
	end
end

function Proxxy:PLAYER_TARGET_CHANGED()
	local cp = GetComboPoints("player", "target")
	if (cp > 0) then
		_,_,sIcon = GetSpellInfo(48889)
		self:NewCounterBar("Combo Points", "Combo Points", cp, 5, cpColor, sIcon)
	else
		local timerbars = self:GetBars()
		if (timerbars["Combo Points"] ~= nil) then
			timerbars["Combo Points"]:destroy()
		end
	end
end

function Proxxy:CheckBuffs()
	for i = 1, 40 do
		local name,rank,icon,count,dtype,dur,expr,caster,isStealable,shouldConsolidate,spellId=UnitAura("player",i)
		if not name then break end
		--print(name .. " dur = " .. dur .. " expr = " .. expr .. " GetTime() = " .. GetTime())

		if (procs[name] ~= nil) or (procs[tostring(spellId)] ~= nil) then
			local spell = procs[name] or procs[tostring(spellId)]
			local spellname = spell.name or name
			local sdb = self.db.profile.spells
			if (spell.ptype ~= nil) and (spell.ptype == "custom") then
				sdb = self.db.profile.custom
			end
			
			if ((sdb[spellname] == nil) or (sdb[spellname].enabled == nil) or (sdb[spellname].enabled == true))
			then
				if dur and dur > 0 then
					self:NewTimerBar(spellId,
						--name .. (rank and rank ~= "" and " (" .. rank .. ")" or "") .. (count and count > 0 and " (" .. count .. (procs[name].maxc ~= nil and "/"..procs[name].maxc..")" or ")") or ""),
						spellname .. (count and count > 0 and " (" .. count .. (spell.maxc ~= nil and "/"..spell.maxc..")" or ")") or ""),
						dur, expr, spell.color or {1,0,0}, icon, "aura"
					)
				else -- setup a non-timer bar here	-- todo: keep calling it over and over and over?
					self:NewCounterBar(spellId,
						--name .. (rank and rank ~= "" and " (" .. rank .. ")" or ""),
						spellname,
						(count or 0), --count and count > 0 and count or 0)
						spell.maxc or 0, --max count (currently no way to get a max that i know of)
						spell.color or {1,0,0}, icon
					)
				end
			end
		end
	end
	
	-- check to see if a timer needs to be removed
	local timerbars = self:GetBars()
	for k,v in pairs(timerbars) do
		local foundSpell = false
		if (v.type == "aura") then
			for i = 1, 40 do
				local name,rank,icon,count,dtype,dur,expr,caster,isStealable,shouldConsolidate,spellId=UnitAura("player",i)
				if not name then break end
				if (v.name == spellId) then
					foundSpell = true
					break
				end
			end

			if (foundSpell == false) and (v.name ~= "Combo Points") then
				v:destroy()
			end
		end
	end
end
