--[[
	Copyright 2010 Matthew Mullins (aka Baffler)
	
    This file is part of Proxxy (World of Warcraft addon).

    Proxxy is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Proxxy is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Proxxy.  If not, see <http://www.gnu.org/licenses/>.
]]--

if not Proxxy then return end
local Proxxy = Proxxy
local media = LibStub("LibSharedMedia-3.0")

local Quartz3 = LibStub("AceAddon-3.0"):GetAddon("Quartz3")
local Player = Quartz3:GetModule("Player")
local useQuartz = false
local QP

local anchor
local db
local def_bar_width
local def_bar_height
local def_font_size
local def_font_type --= media:Fetch('font', "Sans Demibold")
local def_gap_size
local def_alpha_value
local def_text_color
local barCount = 0
local bars = {}


local function UpdateAnchorCords(frame)
	frame:StopMovingOrSizing()
	
	local point, relativeTo, relPoint, xOfs, yOfs = frame:GetPoint()
	--relativeTo = relativeTo:GetName()
	xOfs = floor(xOfs + 0.5)
	yOfs = floor(yOfs + 0.5)
	
	db.profile.Options.AnchorPoint = point
	db.profile.Options.AnchorRelPoint = relPoint
	db.profile.Options.AnchorXOfs = xOfs
	db.profile.Options.AnchorYOfs = yOfs

	frame.text:SetText(point.." "..relPoint.." "..xOfs.." "..yOfs)
end


function Proxxy:SetupAnchor()
	db = Proxxy.db
	def_bar_width = db.profile.Options.BarWidth
	def_bar_height = db.profile.Options.BarHeight
	def_font_size = db.profile.Options.FontSize
	def_font_type = db.profile.Options.FontType
	def_bar_texture = db.profile.Options.BarTexture
	def_gap_size = db.profile.Options.GapSize
	def_alpha_value = db.profile.Options.AlphaValue
	def_text_color = db.profile.Options.TextColor
	
	anchor = CreateFrame('StatusBar', nil, UIParent)
	--barFrame:SetScript('OnHide', OnHide)
	--barFrame:SetScript('OnUpdate', barOnUpdate)
	anchor:SetMovable(true)
	anchor:EnableMouse(true)
	anchor:SetScript("OnMouseDown", anchor.StartMoving)
	anchor:SetScript("OnMouseUp", UpdateAnchorCords)
	--anchor:SetScript("OnUpdate", UpdateAnchorCords)

	anchor:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16})
	anchor:SetBackdropColor(0,0,0)
	anchor.text = anchor:CreateFontString(nil, 'OVERLAY')
	
	anchor.text:SetFont(media:Fetch('font', def_font_type), def_font_size)
	anchor.text:SetShadowColor( 0, 0, 0, 1)
	anchor.text:SetShadowOffset( 0.8, -0.8 )
	anchor.text:SetTextColor(def_text_color[1],def_text_color[2],def_text_color[3])
	anchor.text:SetText("Anchor")
	anchor.text:Show()
	anchor.text:ClearAllPoints()
	anchor.text:SetPoint("LEFT", anchor, "LEFT", 2, 0)
	anchor.text:SetJustifyH("RIGHT")
	
	anchor:Show()
	anchor:ClearAllPoints()
	local tex = media:Fetch("statusbar", def_bar_texture)
	anchor:SetStatusBarTexture(tex)
	anchor:SetWidth(def_bar_width)
	anchor:SetHeight(def_bar_height)
	--
		local aPoint = db.profile.Options.AnchorPoint or "CENTER"
		local relPoint = db.profile.Options.AnchorRelPoint or "CENTER"
		local aXOfs = db.profile.Options.AnchorXOfs or 0
		local aYOfs = db.profile.Options.AnchorYOfs or 0
		anchor:SetPoint(aPoint, UIParent, relPoint, aXOfs, aYOfs)
	--
	anchor:SetStatusBarColor(0,0,0)
	anchor:SetAlpha(1)
	--
	
	self:SetInitialAnchorPoint()
end

function Proxxy:SetAllWidth(value)
	def_bar_width = value
	anchor:SetWidth(value)
	for k,v in pairs(bars) do
		v.barFrame:SetWidth(value)
	end
end

function Proxxy:SetAllHeight(value)
	def_bar_height = value
	anchor:SetHeight(value)

	for k,v in pairs(bars) do
		v.barFrame:SetHeight(value)
		v.barFrame.icon:SetHeight(value-1)
		v.barFrame.icon:SetWidth(value-1)
	end
	self:UpdateBars()
end

function Proxxy:SetAllFontType(value)
	def_font_type = value
	anchor.text:SetFont(media:Fetch('font', value), def_font_size)
	for k,v in pairs(bars) do
		v.barFrame.text:SetFont(media:Fetch('font', value), def_font_size)
		v.barFrame.timetext:SetFont(media:Fetch('font', value), def_font_size)
	end
end

function Proxxy:SetAllFontSize(value)
	def_font_size = value
	anchor.text:SetFont(media:Fetch('font', def_font_type), value)
	for k,v in pairs(bars) do
		v.barFrame.text:SetFont(media:Fetch('font', def_font_type), value)
		v.barFrame.timetext:SetFont(media:Fetch('font', def_font_type), value)
	end
end

function Proxxy:SetAllGapSize(value)
	def_gap_size = value
	self:UpdateBars()
end

function Proxxy:SetAllTexture(value)
	def_bar_texture = value
	local tex = media:Fetch("statusbar", value)
	anchor:SetStatusBarTexture(tex)
	for k,v in pairs(bars) do
		v.barFrame:SetStatusBarTexture(tex)
	end
end

function Proxxy:SetAllAlpha(value)
	def_alpha_value = value
	anchor:SetAlpha(value)
	for k,v in pairs(bars) do
		v.barFrame:SetAlpha(value)
	end
end

function Proxxy:SetAllTextColor(value)
	def_text_color = value
	anchor.text:SetTextColor(value[1],value[2],value[3])
	for k,v in pairs(bars) do
		v.barFrame.text:SetTextColor(value[1],value[2],value[3])
		v.barFrame.timetext:SetTextColor(value[1],value[2],value[3])
	end
end

function Proxxy:HideAnchor(value)
	if value == true then
		anchor:SetAlpha(0)
	else
		anchor:SetAlpha(1)
	end
end
function Proxxy:LockAnchor(value)
	if value == true then
		anchor:SetScript("OnMouseDown", nil)
		anchor:SetScript("OnMouseUp", nil)
	else
		anchor:SetScript("OnMouseDown", anchor.StartMoving)
		anchor:SetScript("OnMouseUp", UpdateAnchorCords)
	end
end
function Proxxy:SetInitialAnchorPoint()
	if (db.profile.Options.AnchorTo == "QUARTZPLAYER") then
		if Quartz3 then
			local anchpos = db.profile.Options.AnchorPOS
			anchor:ClearAllPoints()
			anchor:SetPoint(anchpos, Player.Bar, anchpos, db.profile.Options.AnchorToXoff, db.profile.Options.AnchorToYoff)
			self:HideAnchor(true)
			self:LockAnchor(true)
		end
	end
end
function Proxxy:SetAnchorPoint(value)
	if (value == "FREEUP") then
		db.profile.Options.GrowDir = 1
		
		anchor:ClearAllPoints()
		local aPoint = db.profile.Options.AnchorPoint or "CENTER"
		local relPoint = db.profile.Options.AnchorRelPoint or "CENTER"
		local aXOfs = db.profile.Options.AnchorXOfs or 0
		local aYOfs = db.profile.Options.AnchorYOfs or 0
		anchor:SetPoint(aPoint, UIParent, relPoint, aXOfs, aYOfs)
		anchor:Show()
		self:HideAnchor(false)
		self:LockAnchor(false)
	elseif (value == "FREEDOWN") then
		db.profile.Options.GrowDir = 0
		
		anchor:ClearAllPoints()
		local aPoint = db.profile.Options.AnchorPoint or "CENTER"
		local relPoint = db.profile.Options.AnchorRelPoint or "CENTER"
		local aXOfs = db.profile.Options.AnchorXOfs or 0
		local aYOfs = db.profile.Options.AnchorYOfs or 0
		anchor:SetPoint(aPoint, UIParent, relPoint, aXOfs, aYOfs)
		anchor:Show()
		self:HideAnchor(false)
		self:LockAnchor(false)
	elseif (value == "QUARTZPLAYER") then
		if Quartz3 then
			local anchpos = db.profile.Options.AnchorPOS
			anchor:SetPoint(anchpos, Player.Bar, anchpos, db.profile.Options.AnchorToXoff, db.profile.Options.AnchorToYoff)
			self:HideAnchor(true)
			self:LockAnchor(true)
		end
	end
	self:UpdateBars()
end

function Proxxy:SetAnchorPos(value)
	if (db.profile.Options.AnchorTo == "QUARTZPLAYER") then
		if Quartz3 then
			local oldpos = db.profile.Options.AnchorPOS
			local anchpos = value
			if (anchpos == "TOPLEFT") then
				db.profile.Options.GrowDir = 1
				-- reverse the offsets
				if (oldpos == "TOPRIGHT") or (oldpos == "BOTTOMRIGHT") then
					db.profile.Options.AnchorToXoff = db.profile.Options.AnchorToXoff * -1
				end
				if (oldpos == "BOTTOMRIGHT") or (oldpos == "BOTTOMLEFT") then
					db.profile.Options.AnchorToYoff = db.profile.Options.AnchorToYoff * -1
				end
			elseif (anchpos == "TOPRIGHT") then
				db.profile.Options.GrowDir = 1
				-- reverse the offsets
				if (oldpos == "TOPLEFT") or (oldpos == "BOTTOMLEFT") then
					db.profile.Options.AnchorToXoff = db.profile.Options.AnchorToXoff * -1
				end
				if (oldpos == "BOTTOMRIGHT") or (oldpos == "BOTTOMLEFT") then
					db.profile.Options.AnchorToYoff = db.profile.Options.AnchorToYoff * -1
				end
			elseif (anchpos == "BOTTOMLEFT") then
				db.profile.Options.GrowDir = 0
				-- reverse the offsets
				if (oldpos == "TOPRIGHT") or (oldpos == "BOTTOMRIGHT") then
					db.profile.Options.AnchorToXoff = db.profile.Options.AnchorToXoff * -1
				end
				if (oldpos == "TOPRIGHT") or (oldpos == "TOPLEFT") then
					db.profile.Options.AnchorToYoff = db.profile.Options.AnchorToYoff * -1
				end
			elseif (anchpos == "BOTTOMRIGHT") then
				db.profile.Options.GrowDir = 0
				-- reverse the offsets
				if (oldpos == "TOPLEFT") or (oldpos == "BOTTOMLEFT") then
					db.profile.Options.AnchorToXoff = db.profile.Options.AnchorToXoff * -1
				end
				if (oldpos == "TOPRIGHT") or (oldpos == "TOPLEFT") then
					db.profile.Options.AnchorToYoff = db.profile.Options.AnchorToYoff * -1
				end
			end
			
			anchor:ClearAllPoints()
			anchor:SetPoint(anchpos, Player.Bar, anchpos, db.profile.Options.AnchorToXoff, db.profile.Options.AnchorToYoff)
			anchor:Show()
			self:UpdateBars()
			self:HideAnchor(true)
			self:LockAnchor(true)
		end
	end
end

function Proxxy:SetAnchorOffset()
	if (db.profile.Options.AnchorTo == "QUARTZPLAYER") then
		if Quartz3 then
			local anchpos = db.profile.Options.AnchorPOS
			anchor:SetPoint(anchpos, Player.Bar, anchpos, db.profile.Options.AnchorToXoff, db.profile.Options.AnchorToYoff)
			self:HideAnchor(true)
			self:LockAnchor(true)
		end
	end
end




local function timenum(num)
	if num <= 10 then
		return ('%.1f'):format(num)
	elseif num <= 60 then
		return ('%d'):format(num)
	elseif num <= 3600 then
		return ('%d:%02d'):format(num / 60, num % 60)
	else
		return ('%d:%02d'):format(num / 3600, (num % 3600) / 60)
	end
end

local function barOnUpdate(frame)
	--print("barOnUpdate")
	local timeNOW = GetTime()
	local progress = (frame.self.expires - GetTime())
	if (progress >= 0) then
		frame:SetValue(progress)
		frame.timetext:SetText(timenum(progress))
	else
		-- remove bar?
		frame.self:destroy()
	end
end
	
local function OnHide(frame)
	print("OnHide event fired")
	frame:SetScript('OnUpdate', nil)
end

local timerbar = {}
timerbar.__index = timerbar

function timerbar:newTimer(tName, tText, tDur, tExpr, tColor, tIcon, tType)
	local self = {}
	local timeNOW = GetTime()
	setmetatable(self, timerbar)
	self.name = tName
	self.text = tText
	self.starttime = timeNOW
	self.duration = tDur
	self.expires = tExpr
	self.icon = ticon
	self.type = tType
	
	local barFrame = CreateFrame('StatusBar', nil, UIParent)
	barFrame.self = self
	--barFrame:SetScript('OnHide', OnHide)
	barFrame:SetScript('OnUpdate', barOnUpdate)
	barFrame:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16})
	barFrame:SetBackdropColor(0,0,0)
	barFrame.text = barFrame:CreateFontString(nil, 'OVERLAY')
	barFrame.timetext = barFrame:CreateFontString(nil, 'OVERLAY')
	barFrame.icon = barFrame:CreateTexture(nil, 'DIALOG')
	barFrame.icon:SetTexture(tIcon)
	barFrame.icon:Show()
	barFrame.icon:SetWidth(def_bar_height-1)
	barFrame.icon:SetHeight(def_bar_height-1)
	barFrame.icon:SetTexCoord(0.07, 0.93, 0.07, 0.93)
	barFrame.icon:ClearAllPoints()
	barFrame.icon:SetPoint('RIGHT', barFrame, "LEFT", -1, 0)
	
	barFrame.text:SetFont(media:Fetch('font', def_font_type), def_font_size)
	barFrame.text:SetShadowColor( 0, 0, 0, 1)
	barFrame.text:SetShadowOffset( 0.8, -0.8 )
	barFrame.text:SetTextColor(1,1,1)
	barFrame.text:SetText(tText)
	--text:SetHeight(10)
	barFrame.text:Show()
	barFrame.text:ClearAllPoints()
	barFrame.text:SetPoint("LEFT", barFrame, "LEFT", 2, 0)
	barFrame.text:SetJustifyH("RIGHT")
	
	barFrame.timetext:SetFont(media:Fetch('font', def_font_type), def_font_size)
	barFrame.timetext:SetShadowColor( 0, 0, 0, 1)
	barFrame.timetext:SetShadowOffset( 0.8, -0.8 )
	barFrame.timetext:SetTextColor(1,1,1)
	barFrame.timetext:SetText("0")
	barFrame.timetext:Show()
	barFrame.timetext:ClearAllPoints()
	barFrame.timetext:SetPoint("RIGHT", barFrame, "RIGHT", -2, 0)
	barFrame.timetext:SetJustifyH("RIGHT")
	
	barFrame:SetMinMaxValues(0, tDur)
	barFrame:SetValue(tExpr - timeNOW)
	barFrame:Show()
	barFrame:ClearAllPoints()
	local tex = media:Fetch("statusbar", def_bar_texture)
	barFrame:SetStatusBarTexture(tex)
	barFrame:SetWidth(def_bar_width)
	barFrame:SetHeight(def_bar_height)
	barFrame:SetPoint("CENTER", anchor, "CENTER", 0, ((barCount+1) * (def_bar_height + def_gap_size)))
	barFrame:SetStatusBarColor(tColor[1] or 1, tColor[2] or 0, tColor[3] or 0)
	barFrame:SetAlpha(def_alpha_value)
	
	self.barFrame = barFrame
	bars[tName] = self
	barCount = barCount + 1
	self.index = barCount
	
	return self
end

function timerbar:newCounter(tName, tText, tCount, tMax, tColor, tIcon)
	local self = {}
	setmetatable(self, timerbar)
	self.name = tName
	self.text = tText
	self.count = tCount
	self.max = tMax
	self.icon = ticon
	self.type = "aura"
	
	local barFrame = CreateFrame('StatusBar', nil, UIParent)
	barFrame.self = self
	--barFrame:SetScript('OnHide', OnHide)
	--barFrame:SetScript('OnUpdate', barOnUpdate)
	barFrame:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16})
	barFrame:SetBackdropColor(0,0,0)
	barFrame.text = barFrame:CreateFontString(nil, 'OVERLAY')
	barFrame.timetext = barFrame:CreateFontString(nil, 'OVERLAY')
	barFrame.icon = barFrame:CreateTexture(nil, 'DIALOG')
	barFrame.icon:SetTexture(tIcon)
	barFrame.icon:Show()
	barFrame.icon:SetWidth(def_bar_height-1)
	barFrame.icon:SetHeight(def_bar_height-1)
	barFrame.icon:SetTexCoord(0.07, 0.93, 0.07, 0.93)
	barFrame.icon:ClearAllPoints()
	barFrame.icon:SetPoint('RIGHT', barFrame, "LEFT", -1, 0)
	
	barFrame.text:SetFont(media:Fetch('font', def_font_type), def_font_size)
	barFrame.text:SetShadowColor( 0, 0, 0, 1)
	barFrame.text:SetShadowOffset( 0.8, -0.8 )
	barFrame.text:SetTextColor(1,1,1)
	barFrame.text:SetText(tText)
	--text:SetHeight(10)
	barFrame.text:Show()
	barFrame.text:ClearAllPoints()
	barFrame.text:SetPoint("LEFT", barFrame, "LEFT", 2, 0)
	barFrame.text:SetJustifyH("RIGHT")
	
	barFrame.timetext:SetFont(media:Fetch('font', def_font_type), def_font_size)
	barFrame.timetext:SetShadowColor( 0, 0, 0, 1)
	barFrame.timetext:SetShadowOffset( 0.8, -0.8 )
	barFrame.timetext:SetTextColor(1,1,1)
	if (tMax > 0 or tCount > 0) then barFrame.timetext:SetText(tCount .. (tMax > 0 and "/"..tMax or "")) end
	barFrame.timetext:Show()
	barFrame.timetext:ClearAllPoints()
	barFrame.timetext:SetPoint("RIGHT", barFrame, "RIGHT", -2, 0)
	barFrame.timetext:SetJustifyH("RIGHT")
	
	barFrame:SetMinMaxValues(0, (tMax > 0 and tMax or 1))
	barFrame:SetValue((tMax > 0 and tCount or 1))
	barFrame:Show()
	barFrame:ClearAllPoints()
	local tex = media:Fetch("statusbar", def_bar_texture)
	barFrame:SetStatusBarTexture(tex)
	barFrame:SetWidth(def_bar_width)
	barFrame:SetHeight(def_bar_height)
	barFrame:SetPoint("CENTER", anchor, "CENTER", 0, ((barCount+1) * (def_bar_height + def_gap_size)))
	--barFrame:SetStatusBarColor(0,0,1)
	barFrame:SetStatusBarColor(tColor[1] or 1, tColor[2] or 0, tColor[3] or 0)
	barFrame:SetAlpha(def_alpha_value)
	
	self.barFrame = barFrame
	bars[tName] = self
	barCount = barCount + 1
	self.index = barCount
	
	return self
end

function timerbar:destroy()
	bars[self.name].barFrame:Hide()
	bars[self.name].barFrame:SetParent(nil)
	bars[self.name] = nil
	self = nil
	Proxxy:UpdateBars()
end

function Proxxy:NewTimerBar(name, text, duration, expires, color, icon, stype)
	if (bars[name] == nil) then
		--print ("timerbar:new("..name..", "..text..", "..duration..", "..expires..", "..icon)
		local bar = timerbar:newTimer(name, text, duration, expires, color, icon, stype)
		Proxxy:UpdateBars()
		return bar
	else
		bars[name].expires = expires
		bars[name].barFrame:SetMinMaxValues(0, duration)
		if (text ~= "") then
			bars[name].text = text
			bars[name].barFrame.text:SetText(text)
		end
		Proxxy:UpdateBars()
		return nil
	end
end

function Proxxy:NewCounterBar(name, text, ccount, cmax, color, icon)
	--print ("timerbar:new("..name..", "..text..", "..ccount..", "..cmax..", "..icon)
	if (bars[name] == nil) then
		local bar = timerbar:newCounter(name, text, ccount, cmax, color, icon)
		Proxxy:UpdateBars()
		return bar
	else
		bars[name].count = ccount
		bars[name].text = text
		bars[name].barFrame.text:SetText(text)
		if (cmax > 0 or ccount > 0) then
			bars[name].barFrame.timetext:SetText(ccount .. (cmax > 0 and "/"..cmax or ""))
			bars[name].barFrame:SetValue(ccount)
		end
		Proxxy:UpdateBars()
		return nil
	end
end

function Proxxy:RemoveTimer(spellname)
	--TODO: doesnt really work if the text changes (like if there's (1) charge or something)
	-- but it doesnt matter too much, since a /reloadui would get rid of the timer
	for k,v in pairs(bars) do
		if (string.lower(v.text) == string.lower(spellname)) then
			v:destroy()
		end
	end
end

function Proxxy:UpdateBars()
	local function sort_func(a,b)
		if (a and b) and (a.count and b.count) then
			return a.count > b.count
		elseif a and a.count then
			return true
		elseif b and b.count then
			return false
		end
		return a.expires > b.expires
	end
	tmp = { }
	for k,v in pairs(bars) do
		table.insert(tmp, v)
	end
	table.sort(tmp, sort_func)
	
	barCount = 0
	
	if db.profile.Options.GrowDir == 1 then
		for k,v in ipairs(tmp) do
			barCount = barCount + 1
			v.barFrame:SetPoint("CENTER", anchor, "CENTER", 0, (barCount * (def_bar_height + def_gap_size)))
			v.index = barCount
		end
	else
		for k,v in ipairs(tmp) do
			barCount = barCount + 1
			v.barFrame:SetPoint("CENTER", anchor, "CENTER", 0, 0 - (barCount * (def_bar_height + def_gap_size)))
			v.index = barCount
		end
	end
end

function Proxxy:GetBars()
	return bars
end
