--[[
	Copyright 2010 Matthew Mullins (aka Baffler)
	
    This file is part of Proxxy (World of Warcraft addon).

    Proxxy is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Proxxy is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Proxxy.  If not, see <http://www.gnu.org/licenses/>.
]]--

if not Proxxy then return end
local Proxxy = Proxxy
local _, cClass = UnitClass("player")
local media = LibStub("LibSharedMedia-3.0")
local db

function Proxxy:setupOptions(ProcList)
	local config = LibStub("AceConfig-3.0")
	local dialog = LibStub("AceConfigDialog-3.0")
	db = Proxxy.db
	if (db.profile.spells == nil) then db.profile.spells = { } end
	
	local classSpellOptions = {}
	classSpellOptions = {
		type = 'group',
		name = "Procs for "..cClass,
		desc = "Procs for "..cClass,
		args = {},
	}
	for spellid,spell in pairs(ProcList) do
		if (spell.color ~= nil) then -- prevents the combatlog{} table from showing up (or spells we forgot to set color to :)
			classSpellOptions.args[tostring(spellid)] = {
				type = 'group',
				name = spell.name or spellid,
				desc = spell.name or spellid,
				order = 1,
				args = {
					spellToggle = {
						type = 'toggle',
						name = 'Enabled',
						desc = 'Enable/Disable',
						get = function(info)
							if (db.profile.spells[info.arg] ~= nil) then
								if (db.profile.spells[info.arg].enabled ~= nil) then
									return db.profile.spells[info.arg].enabled
								else
									return true
								end
							end
							return true
						end,
						set = function(info, value)
							if (db.profile.spells[info.arg] == nil) then
								db.profile.spells[info.arg] = { enabled = value }
							else
								db.profile.spells[info.arg].enabled = value
							end
						end,
						order = 10,
						arg = tostring(spellid),
					},
					
					--[[
					spellText = {
						type = 'input',
						name = spellid,
						desc = 'name of spell',
						get = function(info, value)
							return db.profile.spells[info.arg].name or info.arg
						end,
						set = function(info, value)
							if (db.profile.spells[info.arg] == nil) then
								db.profile.spells[info.arg] = { name = true }
							else
								db.profile.spells[info.arg].name = v
							end
						end,
						order = 20,
						arg = tostring(spellid),
					}, ]]--
					
					spellColor = {
						type = 'color',
						name = 'Color',
						desc = 'color',
						get = function(info)
							if (db.profile.spells[info.arg] ~= nil) then
								if (db.profile.spells[info.arg].color ~= nil) then
									return unpack(db.profile.spells[info.arg].color)
								else
									return unpack(ProcList[info.arg].color)
								end
							end
							return unpack(ProcList[info.arg].color)
						end,
						set = function(info, ...)
							if (db.profile.spells[info.arg] == nil) then
								db.profile.spells[info.arg] = { color = {...} }
							else
								db.profile.spells[info.arg].color = {...}
							end
						end,
						order = 30,
						arg = tostring(spellid),
					},
					--[[
					spellSound = {
						type = 'select',
						name = 'Sound',
						desc = 'Sound to make when proc occurs',
						get = getsound,
						set = setsound,
						order = 40,
						values = media,
						arg = tostring(spellid),
					},
					]]--
				},
			}
		end
	end
	
	-- list from the combatlog{} section
	if (ProcList.combatlog ~= nil) then
		for spellid,spell in pairs(ProcList.combatlog) do
			classSpellOptions.args[tostring(spellid)] = {
				type = 'group',
				name = spell.name or spellid,
				desc = spell.name or spellid,
				order = 1,
				args = {
					spellToggle = {
						type = 'toggle',
						name = 'Enabled',
						desc = 'Enable/Disable',
						get = function(info)
							if (db.profile.spells[info.arg] ~= nil) then
								if (db.profile.spells[info.arg].enabled ~= nil) then
									return db.profile.spells[info.arg].enabled
								else
									return true
								end
							end
							return true
						end,
						set = function(info, value)
							if (db.profile.spells[info.arg] == nil) then
								db.profile.spells[info.arg] = { enabled = value }
							else
								db.profile.spells[info.arg].enabled = value
							end
						end,
						order = 10,
						arg = tostring(spellid),
					},
					
					--[[
					spellText = {
						type = 'input',
						name = spellid,
						desc = 'name of spell',
						get = function(info, value)
							return db.profile.spells[info.arg].name or info.arg
						end,
						set = function(info, value)
							if (db.profile.spells[info.arg] == nil) then
								db.profile.spells[info.arg] = { name = true }
							else
								db.profile.spells[info.arg].name = v
							end
						end,
						order = 20,
						arg = tostring(spellid),
					}, ]]--
					
					spellColor = {
						type = 'color',
						name = 'Color',
						desc = 'color',
						get = function(info)
							if (db.profile.spells[info.arg] ~= nil) then
								if (db.profile.spells[info.arg].color ~= nil) then
									return unpack(db.profile.spells[info.arg].color)
								else
									return unpack(ProcList.combatlog[info.arg].color)
								end
							end
							return unpack(ProcList.combatlog[info.arg].color)
						end,
						set = function(info, ...)
							if (db.profile.spells[info.arg] == nil) then
								db.profile.spells[info.arg] = { color = {...} }
							else
								db.profile.spells[info.arg].color = {...}
							end
						end,
						order = 30,
						arg = tostring(spellid),
					},
					--[[
					spellSound = {
						type = 'select',
						name = 'Sound',
						desc = 'Sound to make when proc occurs',
						get = getsound,
						set = setsound,
						order = 40,
						values = media,
						arg = tostring(spellid),
					},
					]]--
				},
			}
		end
	end
	--//////
	
	-- Custom Spell Options
	local cTmpName, cTmpSpellID, cTmpToggle
	local cTmpColor = {0,0,0,1}
	
	local spellCustomOptions = {
		cToggle = {
			type = 'toggle',
			name = 'Enabled',
			desc = 'Enable/Disable',
			get = function(info)
				local sKey = info[1]
				if (db.profile.custom[sKey] ~= nil) then
					if (db.profile.custom[sKey].enabled ~= nil) then
						return db.profile.custom[sKey].enabled
					else
						return true
					end
				end
				return true
			end,
			set = function(info, v)
				local sKey = info[1]
				if (db.profile.custom[sKey] ~= nil) then
					db.profile.custom[sKey].enabled = v
				end
				if (v == false) then
					Proxxy:RemoveTimer(sKey)
				end
			end,
			order = 10,
		},
		
		--cName = {
			--type = 'input',
			--name = 'Name',
			--get = function(info)
				--if (getLeaf(info).arg == db.profile.custom[getLeaf(info).arg].name) then
					--CustomOptions.args[info[1]].args[info[2]].disabled = true
				--else
					--CustomOptions.args[info[1]].args[info[2]].disabled = false
				--end
				--if (db.profile.custom[getLeaf(info).arg] ~= nil) then
					--return db.profile.custom[getLeaf(info).arg].name or ''
				--end
				--return 'deleted'
			--end,
			--set = function(info, v)
				--if (db.profile.custom[getLeaf(info).arg] ~= nil) then
					--db.profile.custom[getLeaf(info).arg].name = v
				--end
			--end,
			--order = 20,
		--}, 
		cColor = {
			type = 'color',
			name = 'color',
			get = function(info)
				local sKey = info[1]
				if (db.profile.custom[sKey] ~= nil) then
					return unpack(db.profile.custom[sKey].color)
				end
				return {1,0,0}
			end,
			set = function(info, ...)
				local sKey = info[1]
				if (db.profile.custom[sKey] ~= nil) then
					db.profile.custom[sKey].color = {...}
				end
			end,
			order = 30,
		},
		cSpellID = {
			type = 'input',
			name = 'Spell ID',
			get = function(info)
				local sKey = info[1]
				if (db.profile.custom[sKey].name ~= nil) then
					return sKey
				end
				return "<not used>"
			end,
			disabled = true,
			order = 40,
		},
		--[[
		cSpellSound = {
			type = 'select',
			name = 'Sound',
			desc = 'Sound to make when proc occurs',
			get = function(info)
				local sKey = info.arg
				return db.profile.custom[sKey].sound or "None"
			end,
			set = setsoundcust,
			order = 40,
			values = media,
		}, ]]--
						
		cDelBtn = {
			type = 'execute',
			name = 'Delete',
			func = function(info)
				local sKey = info[1]
				print("Removed custom proc ("..sKey..")")
				db.profile.custom[sKey] = nil
				CustomOptions.args[sKey] = nil
				ProcList[tostring(sKey)] = nil
			end,
			order = 50,
		},
	}

	function cAddBtnFunc(info)
		if (cTmpToggle == nil) or (cTmpToggle == false) then -- Proc by name only
			if (cTmpName ~= nil) and (cTmpName ~= "") then
				local tmpTbl = {
					color = cTmpColor,
					ptype = 'custom',
				}
				db.profile.custom[cTmpName] = {
					color = cTmpColor,
					ptype = 'custom',
				}
				
				ProcList[cTmpName] = tmpTbl
				
				CustomOptions.args[cTmpName] = {
					type = 'group',
					name = cTmpName,
					desc = cTmpName .. " (useName)",
					arg = tostring(cTmpName),
					order = 100,
					args = spellCustomOptions,
				}
				
				print("Added '"..cTmpName.."'")
			else
				print("Some fields were empty or invalid!")
			end
		else -- proc by Spell ID
			if (cTmpSpellID ~= nil) and (tonumber(cTmpSpellID) > 0) 
				and (cTmpName ~= nil) and (cTmpName ~= "") 
			then
				local tmpTbl = {
					name = cTmpName,
					color = cTmpColor,
					ptype = 'custom',
				}
				db.profile.custom[tostring(cTmpSpellID)] = {
					name = cTmpName,
					color = cTmpColor,
					ptype = 'custom',
				}
				
				--table.insert(spellsWatch, cTmpSpellID, tmpTbl)
				ProcList[tostring(cTmpSpellID)] = tmpTbl
				CustomOptions.args[tostring(cTmpSpellID)] = {
					type = 'group',
					name = cTmpName,
					desc = cTmpName .. " (" .. cTmpSpellID .. ")",
					arg = tostring(cTmpSpellID),
					order = 100,
					args = spellCustomOptions,
				}
				
				print("Added '"..cTmpName.."' ("..cTmpSpellID..")")
			else
				print("Some fields were empty or invalid!")
			end
		end
		
		cTmpName = ""
		cTmpSpellID = "0"
		cTmpToggle = false
		CustomOptions.args[info[1]].args.cSpellID.disabled = false
	end
	
	CustomOptions = {
		type = 'group',
		name = "Custom Procs",
		desc = "Custom Procs",
		args = {},
	}
	
	--Option to add new Proc
	CustomOptions.args["AddNewProc"] = {
		type = 'group',
		name = 'Add Proc',
		desc = 'Add a new proc',
		order = 1,
		args = {
			cName = {
				type = 'input',
				name = 'Name',
				get = function()
					return cTmpName or ''
				end,
				set = function(info, v)
					cTmpName = v
				end,
				usage = '',
				order = 20,
			},
			cColor = {
				type = 'color',
				name = 'color',
				get = function()
					return unpack(cTmpColor)
				end,
				set = function(info, ...)
					cTmpColor = {...}
				end,
				order = 30,
			},
			cToggleSpellID = {
				type = 'toggle',
				name = 'Proc by SpellID',
				desc = 'The proc/buff will be found by the Spell ID only. If you do it this way '..
					'then you will be able to change the name to whatever you like.',
				get = function(info)
					if (cTmpToggle ~= nil) then
						if (cTmpToggle == true) or (cTmpToggle == false) then
							return cTmpToggle
						end
					end
					return false
				end,
				set = function(info, v)
					cTmpToggle = v
					CustomOptions.args[info[1]].args.cSpellID.disabled = not v
				end,
				order = 31,
			},
			cSpellID = {
				type = 'input',
				name = 'Spell ID',
				get = function()
					return cTmpSpellID or "0"
				end,
				set = function(info, v)
					cTmpSpellID = v
				end,
				--validate = function(info, v)
					--print("Test2:: ["..v.."]")
					--return tonumber(v)
				--end,
				usage = '',
				disabled = true,
				order = 40,
			},
			cAddBtn = {
				type = 'execute',
				name = 'Add',
				func = cAddBtnFunc,
				order = 50,
			},
			cResetBtn = {
				type = 'execute',
				name = 'Reset',
				func = function(info)
					cTmpName = ""
					cTmpSpellID = "0"
					cTmpToggle = false
					CustomOptions.args[info[1]].args.cSpellID.disabled = false
				end,
				order = 60,
			},
		},
	}
	
	for spellid,spell in pairs(db.profile.custom) do
		if (spell.ptype == 'custom') then
			CustomOptions.args[tostring(spellid)] = {
				type = 'group',
				name = spellid,
				desc = spellid .. (spell.name and " (" .. spell.name .. ")" or ""),
				arg = tostring(spellid),
				order = 100,
				args = spellCustomOptions,
			}
		end
	end
	-- End Custom Spell Options
	
	
	-- Main Options
	ProxxyMainOptions = {
		name = "Proxxy",
		type = "group",
		args = {
			help = {
				type = "description",
				name = "",
				order = 1,
			},
			barAnchor = {
				type = "select",
				name = "Anchor",
				desc = "What to anchor to",
				get = function(info)
					return db.profile.Options.AnchorTo
				end,
				set = function(info, value)
					db.profile.Options.AnchorTo = value
					Proxxy:SetAnchorPoint(value)
				end,
				values = {["FREEUP"] = "Free (Grow Up)", ["FREEDOWN"] = "Free (Grow Down)", ["QUARTZPLAYER"] = "Quartz Casting Bar",},
				order = 9,
			},
			
			barAnchorPos = {
				type = "select",
				name = "Anchor Position",
				desc = "Where to anchor to",
				get = function(info)
					if (db.profile.Options.AnchorTo ~= "QUARTZPLAYER") then
						ProxxyMainOptions.args.barAnchorPos.disabled = true
					else
						ProxxyMainOptions.args.barAnchorPos.disabled = false
					end
					return db.profile.Options.AnchorPOS
				end,
				set = function(info, value)
					Proxxy:SetAnchorPos(value)
					db.profile.Options.AnchorPOS = value
				end,
				values = {["TOPLEFT"] = "Top Left", ["TOPRIGHT"] = "Top Right", ["BOTTOMLEFT"] = "Bottom Left", ["BOTTOMRIGHT"] = "Bottom Right",},
				order = 11,
			},
			
			anchorXOff = {
				type = "range",
				min = -40,
				max = 40,
				step = 1,
				name = "X Offset Anchor",
				desc = "Offset for anchor on the X-axis (horizontal)",
				get = function(info)
					if (db.profile.Options.AnchorTo ~= "QUARTZPLAYER") then
						ProxxyMainOptions.args.anchorXOff.disabled = true
					else
						ProxxyMainOptions.args.anchorXOff.disabled = false
					end
					return db.profile.Options.AnchorToXoff
				end,
				set = function(info, value)
					if (db.profile.Options.AnchorTo == "QUARTZPLAYER") then
						db.profile.Options.AnchorToXoff = value
						Proxxy:SetAnchorOffset()
					end
				end,
				order = 20,
			},
			anchorYOff = {
				type = "range",
				min = -40,
				max = 40,
				step = 1,
				name = "Y Offset Anchor",
				desc = "Offset for anchor on the Y-axis (vertical)",
				get = function(info)
					if (db.profile.Options.AnchorTo ~= "QUARTZPLAYER") then
						ProxxyMainOptions.args.anchorYOff.disabled = true
					else
						ProxxyMainOptions.args.anchorYOff.disabled = false
					end
					return db.profile.Options.AnchorToYoff
				end,
				set = function(info, value)
					if (db.profile.Options.AnchorTo == "QUARTZPLAYER") then
						db.profile.Options.AnchorToYoff = value
						Proxxy:SetAnchorOffset()
					end
				end,
				order = 21,
			},
			
			barWidth = {
				type = "range",
				min = 125,
				max = 400,
				step = 1,
				name = "Bar Width",
				desc = "How long each bar will be",
				get = function(info)
					return db.profile.Options.BarWidth
				end,
				set = function(info, value)
					db.profile.Options.BarWidth = value
					Proxxy:SetAllWidth(value)
				end,
				order = 22,
			},
			barHeight = {
				type = "range",
				min = 8,
				max = 24,
				step = 1,
				name = "Bar Height",
				desc = "How tall each bar will be",
				get = function(info)
					return db.profile.Options.BarHeight
				end,
				set = function(info, value)
					db.profile.Options.BarHeight = value
					Proxxy:SetAllHeight(value)
				end,
				order = 30,
			},
			barFont = {
				type = 'select',
				dialogControl = 'LSM30_Font',
				name = 'Bar Text Font',
				desc = 'The type of font used for each bar',
				values = media:HashTable("font"),
				get = function()
					return db.profile.Options.FontType
				end,
				set = function(info, value)
					db.profile.Options.FontType = value
					Proxxy:SetAllFontType(value)
				end,
				order = 40,
			},
			barFontSize = {
				type = "range",
				min = 8,
				max = 24,
				step = 1,
				name = "Bar Font Size",
				desc = "Font size used for each bar",
				get = function(info)
					return db.profile.Options.FontSize
				end,
				set = function(info, value)
					db.profile.Options.FontSize = value
					Proxxy:SetAllFontSize(value)
				end,
				order = 50,
			},
			barTexture = {
				type = 'select',
				dialogControl = 'LSM30_Statusbar',
				name = 'Bar Texture',
				desc = 'The type of texture used for each bar',
				values = media:HashTable("statusbar"),
				get = function()
					return db.profile.Options.BarTexture
				end,
				set = function(info, value)
					db.profile.Options.BarTexture = value
					Proxxy:SetAllTexture(value)
				end,
				order = 60,
			},
			barGapSize = {
				type = "range",
				min = 0,
				max = 10,
				step = 1,
				name = "Bar Gap Size",
				desc = "The gap size that will be between each bar",
				get = function(info)
					return db.profile.Options.GapSize
				end,
				set = function(info, value)
					db.profile.Options.GapSize = value
					Proxxy:SetAllGapSize(value)
				end,
				order = 70,
			},
			barAlpha = {
				type = "range",
				min = 0.1,
				max = 1,
				step = 0.1,
				name = "Bar Alpha Value",
				desc = "The opacity for each bar",
				get = function(info)
					return db.profile.Options.AlphaValue
				end,
				set = function(info, value)
					db.profile.Options.AlphaValue = value
					Proxxy:SetAllAlpha(value)
				end,
				order = 80,
			},
			barTextColor = {
				type = "color",
				name = "Bar Font Color",
				desc = "The color of font displayed on each bar",
				get = function(info)
					return unpack(db.profile.Options.TextColor)
				end,
				set = function(info, ...)
					local tcolor = {...}
					db.profile.Options.TextColor = tcolor
					Proxxy:SetAllTextColor(tcolor)
				end,
				order = 90,
			},

		},
	}
	
	config:RegisterOptionsTable("Proxxy-Bliz", ProxxyMainOptions)
	dialog:SetDefaultSize("Proxxy-Bliz", 400, 400)
	Proxxy.optionsFrame = dialog:AddToBlizOptions("Proxxy-Bliz", "Proxxy")
	
	--Class Procs
	config:RegisterOptionsTable("Proxxy-Class", classSpellOptions)
	dialog:SetDefaultSize("Proxxy-Class", 400, 400)
	dialog:AddToBlizOptions("Proxxy-Class", "Class Procs", "Proxxy")
	
	--Custom Procs
	config:RegisterOptionsTable("Proxxy-Custom", CustomOptions)
	dialog:SetDefaultSize("Proxxy-Class", 400, 400)
	dialog:AddToBlizOptions("Proxxy-Custom", "Custom Procs", "Proxxy")
end
